# Generated by Django 4.2.10 on 2024-08-23 06:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0028_alter_moduleconfiguration_module"),
    ]

    operations = [
        migrations.AlterField(
            model_name="interactiveuser",
            name="role_id",
            field=models.IntegerField(blank=True, db_column="RoleID", null=True),
        ),
    ]
