# Generated by Django 3.0.14 on 2022-01-04 11:04
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0015_missing_roles"),
    ]

    operations = [
        migrations.RunSQL(
            "ALTER TABLE tblUsers ADD LastLogin [datetime] NULL"
            if settings.MSSQL else
            'ALTER TABLE "tblUsers" ADD "LastLogin" timestamp NULL',
            reverse_sql='ALTER TABLE "tblUsers" DROP COLUMN "LastLogin"',
        ),
    ]
