# Generated by Django 2.1.11 on 2019-10-04 14:21

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_group_usergroup'),
    ]

    operations = [
        migrations.CreateModel(
            name='FieldControl',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('field', models.CharField(max_length=250, unique=True)),
                ('usage', models.IntegerField(blank=True, max_length=1, null=True)),
                ('module', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='controls', to='core.ModuleConfiguration')),
            ],
            options={
                'db_table': 'core_FieldControl',
            },
        ),
    ]
