# Generated by Django 4.2.20 on 2025-04-04 10:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0030_exportablequerymodel_file_format"),
    ]

    operations = [
        migrations.AlterField(
            model_name="mutationlog",
            name="client_mutation_id",
            field=models.CharField(
                blank=True, db_index=True, max_length=255, null=True
            ),
        ),
    ]
