# Generated by Django 2.1.11 on 2019-10-08 09:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0006_fieldcontrol'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fieldcontrol',
            name='usage',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
