# Generated by Django 3.2.20 on 2023-07-14 08:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0023_alter_jsonext_column_in_tblOfficer'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='usergroup',
            options={'managed': False},
        ),
    ]
