# Generated by Django 3.2.24 on 2024-04-09 10:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0027_alter_interactiveuser_last_login_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='moduleconfiguration',
            name='module',
            field=models.CharField(max_length=80),
        ),
    ]
